/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */

/**
 * @fileoverview This library provides functions for getting skin information.
 */

/**
 * @static
 * @class Provides operations for getting display information about the
 *     currently shown skin.
 * @name gadgets.skins
 */
gadgets.skins = function() {
  var skinProperties = {};

  var requiredConfig = {
    'properties': gadgets.config.ExistsValidator
  };

  gadgets.config.register('skins', requiredConfig, function(config) {
    skinProperties = config['skins']['properties'];
  });


  return {
    /**
     * Override the default properties with a new set of properties.
     *
     * @param {Object} properties The mapping of property names to values.
     */
    init: function(properties) {
      //skinProperties = properties; modified
	  if(properties[gadgets.skins.Property.BG_IMAGE]) document.getElementsByTagName('body')[0].style.backgroundImage = properties[gadgets.skins.Property.BG_IMAGE];
	  if(properties[gadgets.skins.Property.BG_COLOR]) document.getElementsByTagName('body')[0].style.backgroundColor = properties[gadgets.skins.Property.BG_COLOR];
	  if(properties[gadgets.skins.Property.FONT_COLOR]) document.getElementsByTagName('body')[0].style.color = properties[gadgets.skins.Property.FONT_COLOR];
	  if(properties[gadgets.skins.Property.BG_POSITION]) document.getElementsByTagName('body')[0].style.backgroundPosition = properties[gadgets.skins.Property.BG_POSITION];
	  if(properties[gadgets.skins.Property.BG_REPEAT]) document.getElementsByTagName('body')[0].style.backgroundRepeat = properties[gadgets.skins.Property.BG_REPEAT];
	  if(properties[gadgets.skins.Property.ANCHOR_COLOR]) document.linkColor = properties[gadgets.skins.Property.ANCHOR_COLOR];
    },

    /**
     * Fetches the display property mapped to the given key.
     *
     * @param {string} propertyKey The key to get data for;
     *    keys are defined in <a href="gadgets.skins.Property.html"><code>
     *    gadgets.skins.Property.</code></a>
     * @return {string} The data.
     *
     * @member gadgets.skins
     */
    getProperty: function(propertyKey) {
      //return skinProperties[propertyKey] || ''; modified
	  if(propertyKey == gadgets.skins.Property.BG_IMAGE) return document.getElementsByTagName('body')[0].style.backgroundImage;
	  if(propertyKey == gadgets.skins.Property.BG_COLOR) return document.getElementsByTagName('body')[0].style.backgroundColor;
	  if(propertyKey == gadgets.skins.Property.FONT_COLOR) return document.getElementsByTagName('body')[0].style.color;
	  if(propertyKey == gadgets.skins.Property.BG_POSITION) return document.getElementsByTagName('body')[0].style.backgroundPosition;
	  if(propertyKey == gadgets.skins.Property.BG_REPEAT) return document.getElementsByTagName('body')[0].style.backgroundRepeat;
	  if(propertyKey == gadgets.skins.Property.ANCHOR_COLOR) return document.linkColor;
	  return '';
    }
  };
}();
/**
 * @static
 * @class
 * All of the display values that can be fetched and used in the gadgets UI.
 * These are the supported keys for the
 * <a href="gadgets.skins.html#getProperty">gadgets.skins.getProperty()</a>
 * method.
 * @name gadgets.skins.Property
 */
gadgets.skins.Property = gadgets.util.makeEnum([
  /**
   * An image to use in the background of the gadget.
   * @member gadgets.skins.Property
   */
  'BG_IMAGE',

  /**
   * The color of the background of the gadget.
   * @member gadgets.skins.Property
   */
  'BG_COLOR',

  /**
   * The color that the main font should use.
   * @member gadgets.skins.Property
   */
  'FONT_COLOR',

  /**
   * The positioning of the background image
   * @member gadgets.skins.Property
   */
  'BG_POSITION',

  /**
   * The repeat characteristics for the background image
   * @member gadgets.skins.Property
   */
  'BG_REPEAT',

  /**
   * The color that anchor tags should use.
   * @member gadgets.skins.Property
   */
  'ANCHOR_COLOR'
]);
